@@ -21,6 +21,7 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QDesktopServices>
+#include <QSignalSpy>
 
 //qgis includes...
 #include <qgsgeometry.h>
@@ -34,75 +35,48 @@
 #include <qgsproject.h>
 #include <qgssymbol.h>
 #include <qgssinglesymbolrenderer.h>
-//qgis test includes
-#include "qgsrenderchecker.h"
-
-class TestSignalReceiver : public QObject
-{
-    Q_OBJECT
-
-  public:
-    TestSignalReceiver()
-      : QObject( nullptr )
-      , featureBlendMode( QPainter::CompositionMode( 0 ) )
-    {}
-    bool rendererChanged =  false ;
-    QPainter::CompositionMode featureBlendMode;
-    double opacity =  1.0 ;
-  public slots:
-    void onRendererChanged()
-    {
-      rendererChanged = true;
-    }
-    void onFeatureBlendModeChanged( const QPainter::CompositionMode blendMode )
-    {
-      featureBlendMode = blendMode;
-    }
-    void onLayerOpacityChanged( double layerOpacity )
-    {
-      opacity = layerOpacity;
-    }
-};
 
 /**
  * \ingroup UnitTests
  * This is a unit test for the vector layer class.
  */
-class TestQgsVectorLayer : public QObject
+class TestQgsVectorLayer : public QgsTest
 {
     Q_OBJECT
   public:
-    TestQgsVectorLayer() = default;
+    TestQgsVectorLayer()
+      : QgsTest( QStringLiteral( "Vector Renderer Tests" ) ) {}
 
   private:
-    bool mTestHasError =  false ;
-    QgsMapLayer *mpPointsLayer = nullptr;
-    QgsMapLayer *mpLinesLayer = nullptr;
-    QgsMapLayer *mpPolysLayer = nullptr;
+    bool mTestHasError = false;
+    QgsVectorLayer *mpPointsLayer = nullptr;
+    QgsVectorLayer *mpLinesLayer = nullptr;
+    QgsVectorLayer *mpPolysLayer = nullptr;
     QgsVectorLayer *mpNonSpatialLayer = nullptr;
     QString mTestDataDir;
-    QString mReport;
 
 
   private slots:
 
-    void initTestCase(); // will be called before the first testfunction is executed.
+    void initTestCase();    // will be called before the first testfunction is executed.
     void cleanupTestCase(); // will be called after the last testfunction was executed.
-    void init() {} // will be called before each testfunction is executed.
-    void cleanup() {} // will be called after every testfunction.
 
-    void QgsVectorLayerNonSpatialIterator();
-    void QgsVectorLayerGetValues();
-    void QgsVectorLayersetRenderer();
-    void QgsVectorLayersetFeatureBlendMode();
-    void QgsVectorLayersetLayerTransparency();
+    void nonSpatialIterator();
+    void getValues();
+    void setRenderer();
+    void setFeatureBlendMode();
+    void setLayerTransparency();
     void uniqueValues();
     void minimumValue();
     void maximumValue();
+    void minimumAndMaximumValue();
     void isSpatial();
     void testAddTopologicalPoints();
     void testCopyPasteFieldConfiguration();
     void testCopyPasteFieldConfiguration_data();
+    void testFieldExpression();
+    void testFieldAggregateExpression();
+    void testAddFeatureExtentUpdated();
 };
 
 void TestQgsVectorLayer::initTestCase()
@@ -117,67 +91,56 @@ void TestQgsVectorLayer::initTestCase()
   //
   //create a non spatial layer that will be used in all tests...
   //
-  QString myDataDir( TEST_DATA_DIR ); //defined in CmakeLists.txt
+  const QString myDataDir( TEST_DATA_DIR ); //defined in CmakeLists.txt
   mTestDataDir = myDataDir + '/';
-  QString myDbfFileName = mTestDataDir + "nonspatial.dbf";
-  QFileInfo myDbfFileInfo( myDbfFileName );
-  mpNonSpatialLayer = new QgsVectorLayer( myDbfFileInfo.filePath(),
-                                          myDbfFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
+  const QString myDbfFileName = mTestDataDir + "nonspatial.dbf";
+  const QFileInfo myDbfFileInfo( myDbfFileName );
+  mpNonSpatialLayer = new QgsVectorLayer( myDbfFileInfo.filePath(), myDbfFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
   // Register the layer with the registry
   QgsProject::instance()->addMapLayers(
-    QList<QgsMapLayer *>() << mpNonSpatialLayer );
+    QList<QgsMapLayer *>() << mpNonSpatialLayer
+  );
   //
   //create a point layer that will be used in all tests...
   //
-  QString myPointsFileName = mTestDataDir + "points.shp";
-  QFileInfo myPointFileInfo( myPointsFileName );
-  mpPointsLayer = new QgsVectorLayer( myPointFileInfo.filePath(),
-                                      myPointFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
+  const QString myPointsFileName = mTestDataDir + "points.shp";
+  const QFileInfo myPointFileInfo( myPointsFileName );
+  mpPointsLayer = new QgsVectorLayer( myPointFileInfo.filePath(), myPointFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
   // Register the layer with the registry
   QgsProject::instance()->addMapLayers(
-    QList<QgsMapLayer *>() << mpPointsLayer );
+    QList<QgsMapLayer *>() << mpPointsLayer
+  );
 
   //
   //create a poly layer that will be used in all tests...
   //
-  QString myPolysFileName = mTestDataDir + "polys.shp";
-  QFileInfo myPolyFileInfo( myPolysFileName );
-  mpPolysLayer = new QgsVectorLayer( myPolyFileInfo.filePath(),
-                                     myPolyFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
+  const QString myPolysFileName = mTestDataDir + "polys.shp";
+  const QFileInfo myPolyFileInfo( myPolysFileName );
+  mpPolysLayer = new QgsVectorLayer( myPolyFileInfo.filePath(), myPolyFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
   // Register the layer with the registry
   QgsProject::instance()->addMapLayers(
-    QList<QgsMapLayer *>() << mpPolysLayer );
+    QList<QgsMapLayer *>() << mpPolysLayer
+  );
 
 
   //
   // Create a line layer that will be used in all tests...
   //
-  QString myLinesFileName = mTestDataDir + "lines.shp";
-  QFileInfo myLineFileInfo( myLinesFileName );
-  mpLinesLayer = new QgsVectorLayer( myLineFileInfo.filePath(),
-                                     myLineFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
+  const QString myLinesFileName = mTestDataDir + "lines.shp";
+  const QFileInfo myLineFileInfo( myLinesFileName );
+  mpLinesLayer = new QgsVectorLayer( myLineFileInfo.filePath(), myLineFileInfo.completeBaseName(), QStringLiteral( "ogr" ) );
   // Register the layer with the registry
   QgsProject::instance()->addMapLayers(
-    QList<QgsMapLayer *>() << mpLinesLayer );
-
-  mReport += QLatin1String( "<h1>Vector Renderer Tests</h1>\n" );
+    QList<QgsMapLayer *>() << mpLinesLayer
+  );
 }
 
 void TestQgsVectorLayer::cleanupTestCase()
 {
-  QString myReportFile = QDir::tempPath() + "/qgistest.html";
-  QFile myFile( myReportFile );
-  if ( myFile.open( QIODevice::WriteOnly | QIODevice::Append ) )
-  {
-    QTextStream myQTextStream( &myFile );
-    myQTextStream << mReport;
-    myFile.close();
-    //QDesktopServices::openUrl( "file:///" + myReportFile );
-  }
   QgsApplication::exitQgis();
 }
 
-void TestQgsVectorLayer::QgsVectorLayerNonSpatialIterator()
+void TestQgsVectorLayer::nonSpatialIterator()
 {
   QgsFeature f;
   QgsAttributeList myList;
@@ -192,7 +155,7 @@ void TestQgsVectorLayer::QgsVectorLayerNonSpatialIterator()
   QVERIFY( myCount == 3 );
 }
 
-void TestQgsVectorLayer::QgsVectorLayerGetValues()
+void TestQgsVectorLayer::getValues()
 {
   QgsVectorLayer *layer = new QgsVectorLayer( QStringLiteral( "Point?field=col1:real" ), QStringLiteral( "layer" ), QStringLiteral( "memory" ) );
   QVERIFY( layer->isValid() );
@@ -269,69 +232,77 @@ void TestQgsVectorLayer::QgsVectorLayerGetValues()
   delete layer;
 }
 
-void TestQgsVectorLayer::QgsVectorLayersetRenderer()
+void TestQgsVectorLayer::setRenderer()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-  TestSignalReceiver receiver;
-  QObject::connect( vLayer, SIGNAL( rendererChanged() ),
-                    &receiver, SLOT( onRendererChanged() ) );
-  QgsSingleSymbolRenderer *symbolRenderer = new QgsSingleSymbolRenderer( QgsSymbol::defaultSymbol( QgsWkbTypes::PointGeometry ) );
+  const QSignalSpy spy( mpPointsLayer, &QgsVectorLayer::rendererChanged );
 
-  QCOMPARE( receiver.rendererChanged, false );
-  vLayer->setRenderer( symbolRenderer );
-  QCOMPARE( receiver.rendererChanged, true );
-  QCOMPARE( vLayer->renderer(), symbolRenderer );
+  QgsSingleSymbolRenderer *symbolRenderer = new QgsSingleSymbolRenderer( QgsSymbol::defaultSymbol( Qgis::GeometryType::Point ) );
+
+  mpPointsLayer->setRenderer( symbolRenderer );
+  QCOMPARE( spy.count(), 1 );
+  QCOMPARE( mpPointsLayer->renderer(), symbolRenderer );
 }
 
-void TestQgsVectorLayer::QgsVectorLayersetFeatureBlendMode()
+void TestQgsVectorLayer::setFeatureBlendMode()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-  TestSignalReceiver receiver;
-  QObject::connect( vLayer, SIGNAL( featureBlendModeChanged( const QPainter::CompositionMode ) ),
-                    &receiver, SLOT( onFeatureBlendModeChanged( const QPainter::CompositionMode ) ) );
+  const QSignalSpy spy( mpPointsLayer, &QgsVectorLayer::featureBlendModeChanged );
 
-  QCOMPARE( int( receiver.featureBlendMode ), 0 );
-  vLayer->setFeatureBlendMode( QPainter::CompositionMode_Screen );
-  QCOMPARE( receiver.featureBlendMode, QPainter::CompositionMode_Screen );
-  QCOMPARE( vLayer->featureBlendMode(), QPainter::CompositionMode_Screen );
+  mpPointsLayer->setFeatureBlendMode( QPainter::CompositionMode_Screen );
+  QCOMPARE( spy.count(), 1 );
+  QCOMPARE( spy.at( 0 ).at( 0 ).toInt(), static_cast<int>( QPainter::CompositionMode_Screen ) );
+  QCOMPARE( mpPointsLayer->featureBlendMode(), QPainter::CompositionMode_Screen );
+  mpPointsLayer->setFeatureBlendMode( QPainter::CompositionMode_Screen );
+  QCOMPARE( spy.count(), 1 );
+
+  mpPointsLayer->setFeatureBlendMode( QPainter::CompositionMode_Darken );
+  QCOMPARE( spy.count(), 2 );
+  QCOMPARE( spy.at( 1 ).at( 0 ).toInt(), static_cast<int>( QPainter::CompositionMode_Darken ) );
+  QCOMPARE( mpPointsLayer->featureBlendMode(), QPainter::CompositionMode_Darken );
 }
 
-void TestQgsVectorLayer::QgsVectorLayersetLayerTransparency()
+void TestQgsVectorLayer::setLayerTransparency()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-  TestSignalReceiver receiver;
-  QObject::connect( vLayer, &QgsVectorLayer::opacityChanged,
-                    &receiver, &TestSignalReceiver::onLayerOpacityChanged );
+  const QSignalSpy spy( mpPointsLayer, &QgsMapLayer::opacityChanged );
 
-  QCOMPARE( receiver.opacity, 1.0 );
-  vLayer->setOpacity( 0.5 );
-  QCOMPARE( receiver.opacity, 0.5 );
-  QCOMPARE( vLayer->opacity(), 0.5 );
+  mpPointsLayer->setOpacity( 0.5 );
+  QCOMPARE( spy.count(), 1 );
+  QCOMPARE( spy.at( 0 ).at( 0 ).toDouble(), 0.5 );
+  QCOMPARE( mpPointsLayer->opacity(), 0.5 );
+  mpPointsLayer->setOpacity( 0.5 );
+  QCOMPARE( spy.count(), 1 );
+  mpPointsLayer->setOpacity( 1.0 );
+  QCOMPARE( spy.count(), 2 );
+  QCOMPARE( spy.at( 1 ).at( 0 ).toDouble(), 1.0 );
+  QCOMPARE( mpPointsLayer->opacity(), 1.0 );
 }
 
 void TestQgsVectorLayer::uniqueValues()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-
   //test with invalid field
-  QSet<QVariant> values = vLayer->uniqueValues( 1000 );
+  const QSet<QVariant> values = mpPointsLayer->uniqueValues( 1000 );
   QCOMPARE( values.count(), 0 );
 }
 
 void TestQgsVectorLayer::minimumValue()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-
   //test with invalid field
-  QCOMPARE( vLayer->minimumValue( 1000 ), QVariant() );
+  QCOMPARE( mpPointsLayer->minimumValue( 1000 ), QVariant() );
 }
 
 void TestQgsVectorLayer::maximumValue()
 {
-  QgsVectorLayer *vLayer = static_cast< QgsVectorLayer * >( mpPointsLayer );
-
   //test with invalid field
-  QCOMPARE( vLayer->maximumValue( 1000 ), QVariant() );
+  QCOMPARE( mpPointsLayer->maximumValue( 1000 ), QVariant() );
+}
+
+void TestQgsVectorLayer::minimumAndMaximumValue()
+{
+  //test with invalid field
+  QVariant min;
+  QVariant max;
+  mpPointsLayer->minimumAndMaximumValue( 1000, min, max );
+  QCOMPARE( min, QVariant() );
+  QCOMPARE( max, QVariant() );
 }
 
 void TestQgsVectorLayer::isSpatial()
@@ -356,8 +327,8 @@ void TestQgsVectorLayer::testAddTopologicalPoints()
 
   layerLine->startEditing();
   layerLine->addFeature( lineF1 );
-  QgsFeatureId fidLineF1 = lineF1.id();
-  QCOMPARE( layerLine->featureCount(), ( long )1 );
+  const QgsFeatureId fidLineF1 = lineF1.id();
+  QCOMPARE( layerLine->featureCount(), ( long ) 1 );
 
   QCOMPARE( layerLine->undoStack()->index(), 1 );
 
@@ -403,11 +374,11 @@ void TestQgsVectorLayer::testAddTopologicalPoints()
   QVERIFY( nonSpatialLayer->isValid() );
 
   result = nonSpatialLayer->addTopologicalPoints( QgsPoint( 2, 2 ) );
-  QCOMPARE( result, -1 );  // Non editable
+  QCOMPARE( result, -1 ); // Non editable
 
   nonSpatialLayer->startEditing();
   result = nonSpatialLayer->addTopologicalPoints( QgsPoint( 2, 2 ) );
-  QCOMPARE( result, 1 );  // Non spatial
+  QCOMPARE( result, 1 ); // Non spatial
 
   delete nonSpatialLayer;
 
@@ -416,15 +387,15 @@ void TestQgsVectorLayer::testAddTopologicalPoints()
 
   layerPoint->startEditing();
   result = layerPoint->addTopologicalPoints( QgsGeometry() );
-  QCOMPARE( result, 1 );  // Null geometry
+  QCOMPARE( result, 1 ); // Null geometry
 
   delete layerPoint;
 
-  QgsVectorLayer *layerInvalid = new QgsVectorLayer( QStringLiteral(), QStringLiteral( "layer invalid" ), QStringLiteral( "none" ) );
+  QgsVectorLayer *layerInvalid = new QgsVectorLayer( QString(), QStringLiteral( "layer invalid" ), QStringLiteral( "none" ) );
   QVERIFY( !layerInvalid->isValid() );
 
   result = layerInvalid->addTopologicalPoints( QgsPoint( 2, 2 ) );
-  QCOMPARE( result, -1 );  // Invalid layer
+  QCOMPARE( result, -1 ); // Invalid layer
 
   delete layerInvalid;
 }
@@ -448,28 +419,28 @@ void TestQgsVectorLayer::testCopyPasteFieldConfiguration()
   QgsVectorLayer layer1( QStringLiteral( "Point?field=name:string" ), QStringLiteral( "layer1" ), QStringLiteral( "memory" ) );
   QVERIFY( layer1.isValid() );
   QVERIFY( layer1.editorWidgetSetup( 0 ).type().isEmpty() );
-  QCOMPARE( layer1.fieldConfigurationFlags( 0 ), QgsField::ConfigurationFlags() );
+  QCOMPARE( layer1.fieldConfigurationFlags( 0 ), Qgis::FieldConfigurationFlags() );
 
   layer1.setEditorWidgetSetup( 0, QgsEditorWidgetSetup( "ValueMap", QVariantMap() ) );
   QCOMPARE( layer1.editorWidgetSetup( 0 ).type(), QStringLiteral( "ValueMap" ) );
-  layer1.setFieldConfigurationFlags( 0, QgsField::ConfigurationFlag::NotSearchable );
-  QCOMPARE( layer1.fieldConfigurationFlags( 0 ), QgsField::ConfigurationFlag::NotSearchable );
+  layer1.setFieldConfigurationFlags( 0, Qgis::FieldConfigurationFlag::NotSearchable );
+  QCOMPARE( layer1.fieldConfigurationFlags( 0 ), Qgis::FieldConfigurationFlag::NotSearchable );
 
   // export given categories, import all
   QString errorMsg;
   QDomDocument doc( QStringLiteral( "qgis" ) );
-  QgsReadWriteContext context;
+  const QgsReadWriteContext context;
   layer1.exportNamedStyle( doc, errorMsg, context, categories );
   QVERIFY( errorMsg.isEmpty() );
 
   QgsVectorLayer layer2( QStringLiteral( "Point?field=name:string" ), QStringLiteral( "layer2" ), QStringLiteral( "memory" ) );
   QVERIFY( layer2.isValid() );
   QVERIFY( layer2.editorWidgetSetup( 0 ).type().isEmpty() );
-  QCOMPARE( layer2.fieldConfigurationFlags( 0 ), QgsField::ConfigurationFlags() );
+  QCOMPARE( layer2.fieldConfigurationFlags( 0 ), Qgis::FieldConfigurationFlags() );
 
   QVERIFY( layer2.importNamedStyle( doc, errorMsg ) );
   QCOMPARE( layer2.editorWidgetSetup( 0 ).type(), categories.testFlag( QgsMapLayer::Forms ) ? QStringLiteral( "ValueMap" ) : QString( "" ) );
-  QCOMPARE( layer2.fieldConfigurationFlags( 0 ), categories.testFlag( QgsMapLayer::Fields ) ? QgsField::ConfigurationFlag::NotSearchable : QgsField::ConfigurationFlags() );
+  QCOMPARE( layer2.fieldConfigurationFlags( 0 ), categories.testFlag( QgsMapLayer::Fields ) ? Qgis::FieldConfigurationFlag::NotSearchable : Qgis::FieldConfigurationFlags() );
 
   // export all, import given categories
   QDomDocument doc2( QStringLiteral( "qgis" ) );
@@ -479,12 +450,95 @@ void TestQgsVectorLayer::testCopyPasteFieldConfiguration()
   QgsVectorLayer layer3( QStringLiteral( "Point?field=name:string" ), QStringLiteral( "layer3" ), QStringLiteral( "memory" ) );
   QVERIFY( layer3.isValid() );
   QVERIFY( layer3.editorWidgetSetup( 0 ).type().isEmpty() );
-  QCOMPARE( layer3.fieldConfigurationFlags( 0 ), QgsField::ConfigurationFlags() );
+  QCOMPARE( layer3.fieldConfigurationFlags( 0 ), Qgis::FieldConfigurationFlags() );
 
   QVERIFY( layer3.importNamedStyle( doc2, errorMsg, categories ) );
   QCOMPARE( layer3.editorWidgetSetup( 0 ).type(), categories.testFlag( QgsMapLayer::Forms ) ? QStringLiteral( "ValueMap" ) : QString( "" ) );
-  QCOMPARE( layer3.fieldConfigurationFlags( 0 ), categories.testFlag( QgsMapLayer::Fields ) ? QgsField::ConfigurationFlag::NotSearchable : QgsField::ConfigurationFlags() );
+  QCOMPARE( layer3.fieldConfigurationFlags( 0 ), categories.testFlag( QgsMapLayer::Fields ) ? Qgis::FieldConfigurationFlag::NotSearchable : Qgis::FieldConfigurationFlags() );
 }
 
+void TestQgsVectorLayer::testFieldExpression()
+{
+  QgsVectorLayer layer1( QStringLiteral( "Point?field=name:string" ), QStringLiteral( "layer1" ), QStringLiteral( "memory" ) );
+  QVERIFY( layer1.isValid() );
+
+  layer1.addExpressionField( QStringLiteral( "'abc'" ), QgsField( QStringLiteral( "virtual_field" ), QMetaType::Type::QString ) );
+
+  QCOMPARE( layer1.expressionField( layer1.fields().lookupField( QStringLiteral( "virtual_field" ) ) ), QStringLiteral( "'abc'" ) );
+  QCOMPARE( layer1.expressionField( layer1.fields().lookupField( QStringLiteral( "name" ) ) ), QString() );
+}
+
+void TestQgsVectorLayer::testFieldAggregateExpression()
+{
+  QString testPolysFile( TEST_DATA_DIR );
+  testPolysFile += QLatin1String( "/projects/communes.gpkg|layername=communes" );
+
+  QgsVectorLayer layer( testPolysFile, QStringLiteral( "layer1" ), QStringLiteral( "ogr" ) );
+  QVERIFY( layer.isValid() );
+
+  layer.addExpressionField( QStringLiteral( "sum($area)" ), QgsField( QStringLiteral( "virtual_field" ), QMetaType::Type::QString ) );
+
+  const int vfIndex = layer.fields().count() - 1;
+  QCOMPARE( layer.fields().at( 0 ).name(), QStringLiteral( "fid" ) );
+  QCOMPARE( layer.fields().at( vfIndex ).name(), QStringLiteral( "virtual_field" ) );
+
+  QgsFeature feature;
+  auto featureIt = layer.getFeatures();
+  featureIt.nextFeature( feature );
+  QVERIFY( feature.isValid() );
+  QCOMPARE( feature.attribute( 0 ).toInt(), 1 );
+  QVERIFY( qgsDoubleNear( feature.attribute( vfIndex ).toDouble(), 359065580.0, 1 ) );
+
+  QgsFeature feature2;
+  featureIt.nextFeature( feature2 );
+  QVERIFY( feature2.isValid() );
+  QCOMPARE( feature2.attribute( 0 ).toInt(), 2 );
+  QVERIFY( qgsDoubleNear( feature2.attribute( vfIndex ).toDouble(), 359065580.0, 1 ) );
+}
+
+void TestQgsVectorLayer::testAddFeatureExtentUpdated()
+{
+  QgsVectorLayer *layerLine = new QgsVectorLayer( QStringLiteral( "LineString?crs=EPSG:27700" ), QStringLiteral( "layer line" ), QStringLiteral( "memory" ) );
+  QVERIFY( layerLine->isValid() );
+  QCOMPARE( layerLine->featureCount(), static_cast<long>( 0 ) );
+  QCOMPARE( layerLine->extent(), QgsRectangle() );
+
+  const QSignalSpy spyAdded( layerLine, &QgsVectorLayer::featureAdded );
+  const QSignalSpy spyDeleted( layerLine, &QgsVectorLayer::featureDeleted );
+
+  QgsFeature lineF1;
+  lineF1.setGeometry( QgsGeometry::fromWkt( QStringLiteral( "LineString (2 1, 1 1, 1 5, 7 12)" ) ) );
+
+  connect( layerLine, &QgsVectorLayer::featureAdded, this, [&layerLine, &lineF1]( const QgsFeatureId &fid ) {
+    QCOMPARE( fid, lineF1.id() );
+    QCOMPARE( layerLine->extent(), QgsRectangle( 1, 1, 7, 12 ) );
+    QCOMPARE( layerLine->extent3D(), QgsBox3D( 1, 1, std::numeric_limits<double>::quiet_NaN(), 7, 12, std::numeric_limits<double>::quiet_NaN() ) );
+  } );
+
+  connect( layerLine, &QgsVectorLayer::featureDeleted, this, [&layerLine, &lineF1]( const QgsFeatureId &fid ) {
+    QCOMPARE( fid, lineF1.id() );
+    QCOMPARE( layerLine->extent(), QgsRectangle() );
+    QCOMPARE( layerLine->extent3D(), QgsBox3D() );
+  }
+
+  );
+
+  layerLine->startEditing();
+  layerLine->addFeature( lineF1 );
+  QCOMPARE( spyAdded.count(), 1 );
+  QCOMPARE( spyDeleted.count(), 0 );
+  QCOMPARE( layerLine->featureCount(), static_cast<long>( 1 ) );
+  QCOMPARE( spyAdded.at( 0 ).at( 0 ), QVariant( lineF1.id() ) );
+
+  layerLine->deleteFeature( lineF1.id() );
+  QCOMPARE( spyAdded.count(), 1 );
+  QCOMPARE( spyDeleted.count(), 1 );
+  QCOMPARE( layerLine->featureCount(), static_cast<long>( 0 ) );
+  QCOMPARE( spyDeleted.at( 0 ).at( 0 ), QVariant( lineF1.id() ) );
+
+  delete layerLine;
+}
+
+
 QGSTEST_MAIN( TestQgsVectorLayer )
 #include "testqgsvectorlayer.moc"
